use std::io;
use std::path::PathBuf;
use clap::{Args, Parser, Subcommand};

use ark::CompressionType;

/// Intuitive CLI archiving/compression toolkit
#[derive(Debug, Parser)]
#[command(version, about)]
struct Cli {
    #[command(subcommand)]
    op: Command
}

#[derive(Debug, Subcommand)]
enum Command {
    Pack(PackCmd),
    Unpack(UnpackCmd),
}

/// Create an archive
#[derive(Debug, Args)]
struct PackCmd {
    /// Target files/directories to include in archive
    #[arg(required = true)]
    targets: Vec<PathBuf>,

    /// Where to create archive file
    archive_path: PathBuf,

    /// Compression type (inferred from path if not specified)
    #[arg(short, long, value_enum)]
    compression: Option<CompressionType>,
}

/// Extract an archive
#[derive(Debug, Args)]
struct UnpackCmd {
    /// Archive to unpack
    archive_path: PathBuf,

    /// Destionation to unpack into (will be created if necessary)
    #[arg(default_value = ".")]
    output_path: PathBuf,

    /// Compression type (inferred from path if not specified)
    #[arg(short, long, value_enum)]
    compression: Option<CompressionType>,
}

fn main() -> io::Result<()> {
    let cli = Cli::parse();
    match cli.op {
        Command::Pack(pack) => {
            let alg = pack.compression
                .or_else(|| ark::infer_formats(&pack.archive_path))
                .unwrap(); // replace with proper error handling later

            ark::pack_with(pack.targets, pack.archive_path, alg)
        },
        Command::Unpack(unpack) => {
            let alg = unpack.compression
                .or_else(|| ark::infer_formats(&unpack.archive_path))
                .unwrap();

            ark::unpack_with(unpack.archive_path, unpack.output_path, alg)
        },
    }
}
